@@ -49,6 +49,7 @@ class EventSocket(object):
     self._read_event = None
     self._write_event = None
     self._accept_event = None
+    self._connect_event = None
     self._pending_read_cb_event = None
 
     # Cache the peername so we can include it in logs even if the socket
@@ -133,6 +134,9 @@ class EventSocket(object):
     if self._write_event:
       self._write_event.delete()
       self._write_event = None
+    if self._connect_event:
+      self._connect_event.delete()
+      self._connect_event = None
 
     if self._sock:
       self._sock.close()
@@ -213,34 +217,75 @@ class EventSocket(object):
 
     self._accept_event = event.read( self, self._protected_cb, self._accept_cb )
 
-  def connect_blocking(self, *args):
-    """
-    Connect the socket.
-    """
-    self._sock.connect( *args )
-    self._peername = "%s:%d"%self.getpeername()
-    
-    # 7 Aug 09 aaron - Only set these up after a successful connection, else
-    # an error could be raised by the read event that is passed back to the parent
-    # error handler, which can then cause a loop if it assumes that the error is
-    # normally generated by what was an otherwise-functional socket.  Of course
-    # if no connect ever happened then it can't have been a functional socket.
-    self._read_event = event.read( self._sock, self._protected_cb, self._read_cb )
-    self._write_event = event.write( self._sock, self._protected_cb, self._write_cb )
+  def connect(self, *args, **kwargs):
+    '''
+    Connect to the socket. If currently non-blocking, will return immediately
+    and call close_cb when the timeout is reached. If timeout_at is a float,
+    will wait until that time and then call the close_cb. Otherwise, it will
+    set timeout_at as time()+timeout, where timeout is a float argument or the
+    current timeout value of the socket. The check interval for successful
+    connection on a non-blocking socket is 100ms.
 
-  def connect(self, *args):
+    IMPORTANT: If you want the socket to timeout at all in non-blocking mode,
+    you *must* pass in either a relative timout in seconds, or an absolute 
+    value in timeout_at. Otherwise, the socket will forever try to connect. 
+
+    Passes *args on to socket.connect_ex, and **kwargs are used for local
+    control of `timeout` and `timeout_at`.
+    '''
+    timeout_at = kwargs.get('timeout_at')
+    timeout = kwargs.get('timeout')
+    if not isinstance(timeout_at, float):
+      if not isinstance(timeout,(int,long,float)):
+        timeout = self._sock.gettimeout()
+      if timeout>0:
+        timeout_at = time.time()+timeout
+
+    self._connect_cb(timeout_at, *args, immediate_raise=True)
+
+  def _connect_cb(self, timeout_at, *args, **kwargs):
+    '''
+    Local support for synch and asynch connect. Required because 
+    `event.timeout` doesn't support kwargs. They are spec'd though so that
+    we can branch how exceptions are handled. 
+    '''
     err = self._sock.connect_ex( *args )
 
     if not err:
-      self._peername = "%s:%d"%self.getpeername()
+      self._peername = "%s:%d"%self._sock.getpeername()
       self._read_event = event.read( self._sock, self._protected_cb, self._read_cb )
       self._write_event = event.write( self._sock, self._protected_cb, self._write_cb )
+      
+      if self._connect_event:
+        self._connect_event.delete()
+        self._connect_event = None
 
     elif err in (errno.EINPROGRESS,errno.EALREADY):
-      # TODO: track this event and cancel it on close()
-      event.timeout(0.01, self.connect, *args)
+      # Only track timeout if we're about to re-schedule. Should only receive
+      # these on a non-blocking socket.
+      if isinstance(timeout_at,float) and time.time()>timeout_at:
+        self._error_msg = 'timeout connecting to %s'%str(args)
+        self.close()
+        return
+      
+      if self._connect_event:
+        self._connect_event.delete()
+
+      # Checking every 100ms seems to be a reasonable amount of frequency. If
+      # requested this too can be configurable.
+      self._connect_event = event.timeout(0.1, self._connect_cb, 
+        timeout_at, *args)
     else:
-      raise socket.error( err, os.strerror(err) )
+      if self._connect_event:
+        self._connect_event.delete()
+
+      self._error_msg = os.strerror(err)
+      serr = socket.error( err, self._error_msg )
+
+      if kwargs.get('immediate_raise'):
+        raise serr
+      else:
+        self._handle_error( serr )
 
   def set_inactive_timeout(self, t):
     """
